use strict;

sub write_config {
  my($pnp,$cfname) = @_;

  my($datadst);

  my(@namelist) = split('/', $0);
  my($basename) = $namelist[$#namelist];

  if($cfname eq "") {
    $datadst = "|/sbin/isapnp -";
    if(! -x "/sbin/isapnp") {
      return 0;
    }
  }
  else {
    $datadst = ">$cfname";
  }
  if(!open(CONFIG, $datadst)) {
    return 0;
  }
  print CONFIG "# \n";
  print CONFIG "# This configuration file was generated by ";
  print CONFIG "", $basename, ". Do not hand-modify!\n";
  print CONFIG "# PnP data generated by pnpdump v$pnp->{agent_ver}\n";
  print CONFIG "# Compiler flags $pnp->{agent_flags}\n";
  print CONFIG "\n";
  printf CONFIG "(READPORT 0x%lx)\n", $pnp->{readport};
  print CONFIG "(ISOLATE PRESERVE)\n";
  print CONFIG "(IDENTIFY *)\n";
  if($pnp->{verifyld} == 0) {
    print CONFIG "(VERIFYLD N)\n";
  }
  my($i);
  for $i (1 .. $#{$pnp->{board}}) {
    my($board) = $pnp->{board}[$i];
    print CONFIG "\n";
    print CONFIG "# Card $i";
    if($board->{name} ne "") {
      print CONFIG ": " . $board->{name};
    }
    print CONFIG "\n";
    print CONFIG "# vendor id " . $board->{vendorid} .
	", s/n " . $board->{serialnum} . "\n";
    my($j);
    for $j ($board->{first_ld} .. $#{$board->{ld}}) {
      my($ld) = $board->{ld}[$j];
      print CONFIG "#   LD $j";
      if($ld->{name} ne "") {
        print CONFIG ": $ld->{name}";
      }
      print CONFIG "\n";
      print CONFIG "(CONFIGURE " . $board->{vendorid}. "/" .
	$board->{serialnum} . " (LD $j\n";
      my($k);
      for $k (0 .. $#{$ld->{profile}}) {
        my($profile) = $ld->{profile}[$k];
        print CONFIG "#     Profile $k";
        if($k == $ld->{current}{profile}) {
          print CONFIG " (current)";
        }
        print CONFIG ": priority $profile->{priority}\n";
        my($l);
        for $l (0 .. $#{$profile->{irq}}) {
          my($irq) = $profile->{irq}[$l];
          print CONFIG "#       INT $l: " . join(', ', @{$irq->{irqs}});
          print CONFIG ", mode $irq->{mode}\n";
        }
        for $l (0 .. $#{$profile->{dma}}) {
          my($dma) = $profile->{dma}[$l];
          print CONFIG "#       DMA $l: " . join(', ', @{$dma->{dmas}}) . "\n";
        }
        for $l (0 .. $#{$profile->{io}}) {
          my($io) = $profile->{io}[$l];
          printf CONFIG "#       IO $l: minimum %lx, maximum %lx, ",
		$io->{addr_min}, $io->{addr_max};
          print CONFIG "base alignment " . $io->{addr_align} . ", ";
          print CONFIG "window size " . $io->{window_size};
          if($io->{fixed} == 1) {
            print CONFIG ", fixed";
          }
          print CONFIG "\n";
        }
        for $l (0 .. $#{$profile->{mem}}) {
          my($mem) = $profile->{mem}[$l];
          printf CONFIG "#       MEM $l: minimum %lx, maximum %lx, ",
		$mem->{addr_min}, $mem->{addr_max};
          print CONFIG "base alignment " . $mem->{addr_align} . ", ";
          print CONFIG "window size " . $mem->{window_size} . "\n";
        }
      }
      my($current) = $ld->{current};
      for $k (0 .. $#{$current->{irq}}) {
        my($irq) = $current->{irq}[$k];
        print CONFIG "(INT $k (IRQ $irq (MODE " .
		$ld->{profile}[$current->{profile}]{irq}[$k]{mode} . ")))\n";
      }
      for $k (0 .. $#{$current->{dma}}) {
        my($dma) = $current->{dma}[$k];
        print CONFIG "(DMA $k (CHANNEL $dma))\n";
      }
      for $k (0 .. $#{$current->{io}}) {
        my($io) = $current->{io}[$k];
        printf CONFIG "(IO $k (BASE 0x%lx))\n", $io;
      }
      for $k (0 .. $#{$current->{mem}}) {
        my($mem) = $current->{mem}[$k];
        printf CONFIG "(MEM $k (BASE 0x%lx) (MODE br) (UPPER 0x%lx)\n",
		$mem, $ld->{profile}[$current->{profile}]{mem}[$k]{window_size};
      }
      print CONFIG "(ACT Y)))\n\n";
    }
  }
  close(CONFIG);
}

sub read_config {
  my($cfname) = @_;
  my($pnp);

  if($cfname eq "") {
    return -1;
  }
  if(!open(CONFIG,"<$cfname")) {
    return 0;
  }

  $pnp->{boardsfound} = 1;
  $pnp->{verifyld} = 0;

  unless(($_ = <CONFIG>) =~ /^# /) { close(CONFIG); return -2; }
  unless(($_ = <CONFIG>) =~ /^# This configuration file/) {
    close(CONFIG);
    return -2;
  }
  unless(($_ = <CONFIG>) =~ /^# PnP data generated by pnpdump v(\S*)/) {
    close(CONFIG);
    return -2;
  }
  $pnp->{agent_ver} = $1;
  unless(($_ = <CONFIG>) =~ /^# Compiler flags (.*)/) {
    close(CONFIG);
    return -2;
  }
  $pnp->{agent_flags} = $1;
  unless(($_ = <CONFIG>) =~ /^\n/) { close(CONFIG); return -2; }
  unless(($_ = <CONFIG>) =~ /^\(READPORT 0x(\S*)\)/) {
    close(CONFIG);
    return -2;
  }
  $pnp->{readport} = hex($1);
  unless(($_ = <CONFIG>) =~ /^\(ISOLATE .*\)/) { close(CONFIG); return -2; }
  unless(($_ = <CONFIG>) =~ /^\(IDENTIFY \*\)/) { close(CONFIG); return -2; }
  if((($_ = <CONFIG>) =~ /^\(VERIFYLD N\)/)) {
    $pnp->{verifyld} = 0;
    $_ = <CONFIG>;
  }
  unless(/^\n/) { close(CONFIG); return -2; }
  while(($_ = <CONFIG>) =~ /^# Card (\d*): (.*)/ && !eof(CONFIG)) {
    my($brd) = $1;
    my($board) = \%{$pnp->{board}[$brd]};
    $board->{name} = $2;
    unless(($_ = <CONFIG>) =~ /^# vendor id (\S*), s\/n (.*)/) {
      close(CONFIG);
      return -2;
    }
    $board->{vendorid} = $1;
    $board->{serialnum} = $2;
    $board->{first_ld} = -1;
    $_ = <CONFIG>;
    while(/^#\s*LD (\d*)/ && !eof(CONFIG)) {
      my($ldn) = $1;
      my($ld) = \%{$board->{ld}[$ldn]};
      if($board->{first_ld} == -1) { $board->{first_ld} = $ldn; }
      if(/^#\s*LD \d*: (.*)/) { $ld->{name} = $1; }
      unless(($_ = <CONFIG>) =~ 
	/^\(CONFIGURE $board->{vendorid}\/$board->{serialnum} \(LD $ldn/) {
        close(CONFIG);
        return -2;
      }
      $_ = <CONFIG>;
      while(/^#\s*Profile (\d*)(.*): priority (.*)/ && !eof(CONFIG)) {
        my($prof) = $1;
        my($profile) = \%{$ld->{profile}[$prof]};
        $profile->{priority} = $3;
        if($2 =~ /current/) {$ld->{current}{profile} = $prof; }
        $_ = <CONFIG>;
        while(/^#\s*INT (\d*): (.*), mode (.*)/ && !eof(CONFIG)) {
          @{$profile->{irq}[$1]{irqs}} = split(', ', $2);
          $profile->{irq}[$1]{mode} = $3;
          $_ = <CONFIG>;
        }
        while(/^#\s*DMA (\d*): (.*)/ && !eof(CONFIG)) {
          @{$profile->{dma}[$1]{dmas}} = split(', ', $2);
          $_ = <CONFIG>;
        }
        while(/^#\s*IO (\d*): minimum (\S*), maximum (\S*),/ && !eof(CONFIG)) {
          $profile->{io}[$1]{addr_min} = hex($2);
          $profile->{io}[$1]{addr_max} = hex($3);
          /^#\s*IO (\d*): .*, base alignment (\d*), window size (\d*)/;
          $profile->{io}[$1]{addr_align} = $2;
          $profile->{io}[$1]{window_size} = $3;
          if(/^#\s*IO (\d*): .*,fixed/) { $profile->{io}[$1]{fixed} = 1; }
          $_ = <CONFIG>;
        }
        while(/^#\s*MEM (\d*): minimum (\S*), maximum (\S*),/ &&
		!eof(CONFIG)) {
          $profile->{mem}[$1]{addr_min} = hex($2);
          $profile->{mem}[$1]{addr_max} = hex($3);
          /^#\s*MEM (\d*): .*, base alignment (\d*), window size (\d*)/;
          $profile->{mem}[$1]{addr_align} = $2;
          $profile->{mem}[$1]{window_size} = $3;
          $_ = <CONFIG>;
        }
      }
      $ld->{current}{irq} = [];
      while(/^\(INT (\d*) \(IRQ (\d*) / && !eof(CONFIG)) {
        $ld->{current}{irq}[$1] = $2;
        $_ = <CONFIG>;
      }
      $ld->{current}{dma} = [];
      while(/^\(DMA (\d*) \(CHANNEL (\d*)\)\)/ && !eof(CONFIG)) {
        $ld->{current}{dma}[$1] = $2;
        $_ = <CONFIG>;
      }
      $ld->{current}{io} = [];
      while(/^\(IO (\d*) \(BASE 0x(\S*)\)\)/ && !eof(CONFIG)) {
        $ld->{current}{io}[$1] = hex($2);
        $_ = <CONFIG>;
      }
      $ld->{current}{mem} = [];
      while(/^\(MEM (\d*) \(BASE 0x(\S*)\) / && !eof(CONFIG)) {
        $ld->{current}{mem}[$1] = hex($2);
        $_ = <CONFIG>;
      }
      unless(/^\(ACT Y/) {
        close(CONFIG);
        return -2;
      }
      unless(($_ = <CONFIG>) =~ /^\n/) { close(CONFIG); return -2; }
      $_ = <CONFIG>;
    }
  }

  close(CONFIG);

  $pnp;
}

1;
